import db
from flask import *
import logger

db = db.DB()
home_txt = "The CDR project is open-source solution for content disarm and reconstruction at Mailboxes.</br> The CDR " \
           "works by processing all incoming e-mails in a specific network, deconstructing them, and removing the " \
           "elements that do not match the file type's standards or set policies. CDR technology then rebuilds the " \
           "e-mail into clean versions that can be sent on to end users as intended "
home_title = "Welcome to the CDR Portal"
header_txt = "CDR Portal"
policy_list = db.fetch_policies()

app = Flask(__name__)


def login_check():
    return db.is_logged_in


@app.route('/index')
def index():
    return render_template("message.html", title=home_title, text=home_txt, header=header_txt)


@app.route('/logout', methods=('GET', 'POST'))
def logout():
    db.is_logged_in = False
    return render_template('logout.html')


@app.route('/', methods=('GET', 'POST'))
def root():
    if request.method == 'POST':
        username, password = request.form['username'], request.form['password']
        if db.login(username, password):
            return render_template("message.html", title=home_title, text=home_txt, header=header_txt)
        else:
            error = "Wrong username or password."
            return render_template('login.html', error=error)
    return render_template('login.html')


@app.route('/mailbox_edit', methods=('GET', 'POST'))
def mailbox_edit():
    if not login_check():
        return redirect('/')
    if request.method == 'GET':
        data = db.fetch_mailboxes_with_policy()
        return render_template('mailbox_edit.html', title="CDR-Edit Mailbox", data=data)
    else:
        mail_id = request.form['id']
        data = db.fetch_mailbox_data(mail_id)
        return render_template('mailbox_edit_form.html', data=data, title='CDR-Edit Mailbox')


@app.route('/mailbox_edit_form', methods=('GET', 'POST'))
def mailbox_edit_form():
    if request.method == 'POST':
        firstname = request.form['firstname']
        lastname = request.form['lastname']
        role = request.form['role']
        mailbox_name = request.form['mailbox_name']
        mailbox_id = request.form['id']
        title= "Mailbox edit"
        desc = "Edit mailbox id "+mailbox_id+ " to Firstname: "+ firstname+", Lastname: "+lastname+", Mailbox: "+mailbox_name+", Role: "+role+". "
        try:
            db.update_mailbox_info(mailbox_id, firstname, lastname, mailbox_name, role)
            text = 'Change successful'
            desc+= text
            logger.logger_events(title, desc)
            return render_template('message.html', title=title, text=text)
        except:
            text = 'An Error occurred'
            desc += text
            logger.logger_events(title, desc)
            return render_template('message.html', title=title, text=text)


@app.route('/policy_all', methods=('GET', 'POST'))
def policy_all():
    if not login_check():
        return redirect('/')
    if request.method == 'GET':
        return render_template('policy_all.html', title='General policy edit', data=policy_list)
    if request.method == 'POST':
        policy = request.form['check']
        title = "Policy Edit for all mailboxes"
        desc = title + " - change policy to: " + policy
        try:
            db.update_policy_all(policy)
            desc += " succeed"
            logger.logger_events(title, desc)
            return render_template('message.html', title="General Policy Edit", text='Changed Successful')
        except:
            desc += " failed"
            logger.logger_events(title, desc)
            return render_template('message.html', title="General Policy Edit", text='Error occurred')


@app.route('/policy_one', methods=('GET', 'POST'))
def policy_one():
    if not login_check():
        return redirect('/')
    if request.method == 'GET':
        data = db.fetch_mailboxes_with_policy()
        return render_template('policy_one.html', title='Edit mailbox policy', data=data)
    else:
        mail_id = request.form['id']
        return render_template('policy_one_form.html', id=mail_id, title="Choose wanted policy", data=policy_list)


@app.route('/policy_one_form', methods=('GET', 'POST'))
def policy_one_form():
    if not login_check():
        return redirect('/')
    if request.method == 'POST':
        policy = request.form['check']
        mailbox_id = request.form['id']
        title = "Policy Edit"
        desc = title + " for mailboxID: " + mailbox_id + " change policy to: " + policy
        try:
            db.update_policy_one(mailbox_id, policy)
            desc += " succeed."
            logger.logger_events(title, desc, mailbox_id)
            return render_template('message.html', title=title, text='Change Successful')
        except:
            desc += " failed."
            logger.logger_events(title, desc, mailbox_id)
            return render_template('message.html', title=title, text='Error occurred')


@app.route('/about', methods=('GET', 'POST'))
def about():
    if not login_check():
        return redirect('/')
    return render_template('about.html', title="About")


@app.route('/mailbox_add', methods=('GET', 'POST'))
def mailbox_add():
    if not login_check():
        return redirect('/')
    if request.method == 'POST':
        firstname = request.form['firstname']
        lastname = request.form['lastname']
        role = request.form['role']
        mailbox_name = request.form['mailbox_name']
        title = "Mailbox add"
        try:
            db.add_mailbox(firstname, lastname, mailbox_name, role)
            text = 'Created successfully'
            desc = title + " " + mailbox_name + " " + text
            logger.logger_events(title, desc)
            return render_template('message.html', title=title, text=text)
        except:
            text = 'An Error occurred'
            desc = title + " " + mailbox_name + " " + text
            logger.logger_events(title, desc)
        return render_template('message.html', title=title, text=text)
    else:
        return render_template('mailbox_add.html', title="Add new Mailbox")


@app.route('/delete_mailbox', methods=('GET', 'POST'))
def delete_mailbox():
    if not login_check():
        return redirect('/')
    if request.method == 'GET':
        mailbox_data = db.fetch_mailboxes_with_policy()
        return render_template('delete_mailbox.html', title='Delete Mailbox', data=mailbox_data)
    if request.method == 'POST':
        mail_id = request.form['id']
        title = "Delete Mailbox"
        try:
            db.delete_mailbox(mail_id)
            text = 'Delete Successful'
            desc = title + " id: " + mail_id + " " + text
            logger.logger_events(title, desc)
            return render_template('message.html', title=title, text=text)
        except:
            text = 'An Error occurred'
            desc = title + " id: " + mail_id + " " + text
            logger.logger_events(title, desc)
            return render_template('message.html', title=title, text=text)


@app.route('/reports', methods=('GET', 'POST'))
def reports():
    if not login_check():
        return redirect('/')
    if request.method == 'GET':
        data = db.fetch_reports()
        return render_template('reports.html', title="Show reports", data=data)


@app.route('/logs', methods=('GET', 'POST'))
def logs():
    if not login_check():
        return redirect('/')
    if request.method == 'GET':
        data = db.fetch_logs( )
        return render_template('logs.html', title="Show logs", data=data)


if __name__ == '__main__':
    app.run(host='0.0.0.0')




import time, datetime
import logger
import mail_handler

last_mail_uid = 166
delay = 5
last_time_checked = datetime.datetime.now()
mail = mail_handler.MailHandler()

while True:
    message = mail.fetch_mails()
    for m in message:
        uid = str(int(m[0]))
        title = "Got new mail id:" + uid
        logger.logger_logs(title, "start processing.")
        log = mail.process_mail(m)
        logger.logger_logs("CDR action taken", log)
        logger.logger_logs("Mail id:" + uid + " delivered", "CDR completed the processing.")
    mail.clear_mailbox()
    time.sleep(delay)



import db
from _datetime import datetime

log_db = db.DB()


def logger_events(title, desc, mail_id=0):
    if title and desc:
        now = datetime.now()
        date_time = now.strftime("%m/%d/%Y, %H:%M:%S")
        log_db.add_log_event(title, desc, mail_id, date_time)
        return True
    else:
        return False


def logger_logs(title, details):
    if title and details:
        now = datetime.now()
        title = now.strftime("%m/%d/%Y, %H:%M:%S") + " : " + title
        log_db.add_log(title, details)
        return True
    else:
        return False



import mysql.connector
import os

HOST = os.environ.get('DB_URL', '127.0.0.1')
DB_USER = os.environ.get('DB_USER', 'root')
DB_PASS = os.environ.get('DB_PASS', '123456')
os.environ['LOGGED'] = 'False'


class DB:
    def __init__(self):
        self.connection = mysql.connector.connect(user=DB_USER,
                                                  password=DB_PASS, host=HOST, database='cdr')
        self.cursor = self.connection.cursor(buffered=True)
        self.dic_cur = self.connection.cursor(dictionary=True, buffered=True)
        self.is_logged_in = os.environ['LOGGED']

    def login(self, user, password):
        if self.is_logged_in:
            return True
        else:
            self.cursor.execute('SELECT username,password FROM admin')
            admin = self.cursor.fetchone()
            if user == admin[0] and password == admin[1]:
                self.is_logged_in = True
                os.environ['LOGGED'] = 'True'
                return True
            else:
                return False

    def fetch_policies(self):
        self.cursor.execute('SELECT * FROM policy')
        return self.cursor.fetchall()

    def fetch_mailboxes(self):
        self.cursor.execute('SELECT * FROM mailbox')
        return self.cursor.fetchall()

    def fetch_mailbox_data(self, mail_id):
        self.cursor.execute('SELECT * FROM mailbox where Id=' + mail_id)
        return self.cursor.fetchone()

    def fetch_mailbox_data_dic(self, mail_id):
        self.dic_cur.execute('SELECT * FROM mailbox where Id=' + mail_id)
        return self.dic_cur.fetchone()

    def fetch_mailbox_name(self, mail_id):
        self.cursor.execute('SELECT FirstName,LastName FROM mailbox where Id=' + mail_id)
        return self.cursor.fetchone()

    def update_policy_all(self, policy_id):
        self.cursor.execute("UPDATE cdr.mailbox SET PolicyID = '" + policy_id + "';")
        self.connection.commit()

    def update_policy_one(self, mail_id, policy_id):
        self.cursor.execute("UPDATE cdr.mailbox SET PolicyID = '" + policy_id + "' WHERE ID ='" + mail_id + "';")
        self.connection.commit()

    def update_mailbox_info(self, mail_id, first_name, last_name, mailbox_name, role):
        self.cursor.execute("UPDATE cdr.mailbox SET FirstName ='" + first_name + "', LastName = '" + last_name +
                            "',MailBoxName = '" + mailbox_name + "' , Role = '" + role +
                            "' WHERE(ID = '" + mail_id + "');")
        self.connection.commit()

    def add_mailbox(self, first_name, last_name, mailbox_name, role):
        self.cursor.execute("INSERT INTO `cdr`.`mailbox` (`FirstName`, `LastName`, `MailBoxName`, `Role`,"
                            " `PolicyID`) VALUES ('" + first_name + "', '" + last_name + "', '" + mailbox_name +
                            "', '" + role + "', '1');")
        self.connection.commit()

    def delete_mailbox(self, mailbox_id):
        self.cursor.execute("DELETE FROM cdr.mailbox WHERE (ID='" + mailbox_id + "');")
        self.connection.commit()

    def fetch_reports(self):
        self.dic_cur.execute('SELECT * FROM events')
        ret = self.dic_cur.fetchall()
        self.connection.commit()
        return ret

    def fetch_logs(self):
        self.dic_cur.execute('SELECT * FROM logs')
        ret = self.dic_cur.fetchall()
        self.connection.commit()
        return ret

    def fetch_mailboxes_dic(self):
        self.dic_cur.execute('SELECT * FROM mailbox')
        return self.dic_cur.fetchall()

    def fetch_mailboxes_with_policy(self):
        data = self.fetch_mailboxes_dic()
        policy = self.fetch_policies()
        for i in range(len(data)):
            data[i]['PolicyID'] = policy[data[i]['PolicyID'] - 1][1]
        return data

    def add_log_event(self, title, description, mail_id, date):
        mail_id = str(mail_id)
        self.cursor.execute("INSERT INTO `cdr`.`events` (`title`, `details`, `mailboxid`, `date`) "
                            "VALUES ('" + title + "', '" + description + "', '" + mail_id +
                            "', '" + date + "');")
        self.connection.commit()

    def add_log(self, title, details):
        txt = "INSERT INTO `cdr`.`logs` (`title`, `details`) VALUES ('" + title + "', '" + details + "');"
        self.cursor.execute(txt)
        self.connection.commit()

    def get_mailbox_by_mail(self, mailbox):
        txt = 'SELECT * FROM mailbox where MailBoxName="' + mailbox + '";'
        self.dic_cur.execute(txt)
        return self.dic_cur.fetchone()

    def close(self):
        self.connection.close()

    def clear(self):
        self.connection.commit();


import os
import json
from imbox import Imbox
import db
import base64
from bs4 import BeautifulSoup
import smtplib
from email.message import EmailMessage
from email.mime.application import MIMEApplication

ADMIN_MAIL = os.environ.get('ADMIN_MAIL', 'CDR@localhost')
PASSWORD = os.environ.get('ADMIN_PASS', '123456')
SERVER = os.environ.get('MAIL_SERVER', "localhost")
BAD_FILETYPES = ['pdf', 'exe', 'doc', 'xls', 'vbs', 'jpeg', 'zip', 'rtf', 'scr']

is_gt = lambda x, y: x if x > y else y


def get_policy_actions_by_id(policy_id):
    actions = {
        "url_remove": False,
        "attachments_remove": False,
        "block_mail": False,
        "add_alert": False,
        "filtering": False
    }
    if policy_id == 0:
        return actions
    if policy_id <= 5:
        actions["add_alert"] = True
    if 4 >= policy_id >= 3:
        actions["filtering"] = True
    if policy_id <= 2:
        actions["attachments_remove"] = True
    if policy_id <= 3:
        actions["url_remove"] = True
    if policy_id == 1:
        actions["block_mail"] = True
    return actions


def get_content(message):
    if message:
        msg = message[1]
        return_msg = {
            'attachments': msg.attachments,
            'body': msg.body,
            'date': msg.parsed_date,
            'headers': msg.headers,
            'sent_to': msg.sent_to,
            'sent_from': msg.sent_from,
            'subject': msg.subject
        }
        return return_msg
    return None


def add_alert(mail_content):
    mail_content['body']['plain'].append(
        "\n This mail has been scanned by CDR - Don't open links and attachments from unknown senders.")
    soup = BeautifulSoup(mail_content['body']['html'][0], 'html.parser')
    tag = soup.new_tag('div')
    tag.string = "This mail has been scanned by CDR - Don't open links and attachments from unknown senders."
    soup.body.append(tag)
    mail_content['body']['html'][0] = str(soup)


def remove_attachment(mail_content, filter=False):
    if not filter:
        mail_content['attachments'].clear()
    else:
        for att in mail_content['attachments']:
            if (att['filename'].split('.')[-1] in BAD_FILETYPES):
                mail_content['attachments'].remove(att)


def filter_content(mail_content):
    path = os.path.join(os.path.dirname(__file__), 'static', 'url_blocklist.json')
    f_url = open(path, 'r')
    blocklist = json.load(f_url)
    f_url.close()
    soup = BeautifulSoup(mail_content['body']['html'][0], 'html.parser')
    for a in soup.findAll('a', href=True):
        for url in blocklist:
            if (a.get('href') == url['url']):
                a.extract()
    mail_content['body']['html'][0] = str(soup)


def remove_urls(mail_content):
    soup = BeautifulSoup(mail_content['body']['html'][0], 'html.parser')
    for a in soup.findAll('a', href=True):
        a.extract()
    mail_content['body']['html'][0] = str(soup)


def format_mail_to_send(mail_content):
    out_message = EmailMessage()
    out_message.add_header("cdrApproved", "True")
    out_message["Subject"] = mail_content['subject']
    out_message["From"] = mail_content['sent_from'][0]["email"]
    out_message["To"] = mail_content['sent_to'][0]["email"]
    text = mail_content['body']["plain"][0]
    html = mail_content['body']["html"][0]
    out_message.set_content(html, subtype='html')

    for idx, attachment in enumerate(mail_content['attachments']):
        try:
            data = MIMEApplication(attachment.get('content').read())
            file_name = attachment.get('filename')
            maintype, subtype = attachment['content-type'].split('/', 1)
            dec_data = base64.b64decode(data.get_payload())
            full_path = os.path.join(os.path.dirname(__file__), file_name)
            fb = open(full_path, "wb")
            fb.write(dec_data)
            fb.close()
            fa = open(full_path, "rb")
            out_message.add_attachment(fa.read(), maintype=maintype, subtype=subtype, filename=file_name)
            fa.close()
            os.remove(full_path)
        except:
            print('problem with attachment')
    return out_message


def send_mail(mail_content):
    with smtplib.SMTP(SERVER, 25) as smtp_server:
        smtp_server.login(ADMIN_MAIL, PASSWORD)
        out_message = format_mail_to_send(mail_content)
        smtp_server.sendmail(
            mail_content['sent_from'][0]["email"], mail_content['sent_to'][0]["email"], out_message.as_string()
        )


def exec_policy(actions, mail_content):
    log_text = ""
    if actions['block_mail']:
        return "Mail was blocked due to policy."
    if actions["attachments_remove"]:
        remove_attachment(mail_content)
        log_text += "Attachments were removed. "
    if actions["filtering"]:
        filter_content(mail_content)
        remove_attachment(mail_content, True)
        log_text += "Attachments and urls were filtered. "
    if actions["url_remove"]:
        remove_urls(mail_content)
        log_text += "Urls were removed. "
    if actions["add_alert"]:
        add_alert(mail_content)
        log_text += "Alert was added to the mail. "
    send_mail(mail_content)
    return log_text


class MailHandler:
    def __init__(self):
        self.mail = Imbox(SERVER, ADMIN_MAIL, PASSWORD, False)
        self.last_uid = 0
        self.db = db.DB()

    def fetch_mails(self):
        mails = self.mail.messages()
        for message in mails:
            # print(message)
            self.last_uid = is_gt(self.last_uid, int(message[0]))
        return mails

    def clear_mailbox(self):
        uid = '*:' + str(self.last_uid)
        for uid, massage in self.mail.messages(uid__range=uid):
            print(uid)
            self.mail.delete(uid)

    def get_policy_for_mailbox(self, mailbox):
        self.db.clear()
        data = self.db.get_mailbox_by_mail(mailbox)
        if data:
            return data['PolicyID']
        return 0

    def process_mail(self, message):
        if type(message) is tuple:
            sent_to = message[1].sent_to[0]['email']
            policy_id = self.get_policy_for_mailbox(sent_to)
            actions = get_policy_actions_by_id(policy_id)
            mail_content = get_content(message)
            log = "From: " + mail_content['sent_from'][0]["email"] + " Sent to:" + mail_content['sent_to'][0][
                "email"] + ": "
            log = log + exec_policy(actions, mail_content)
            return log
